import logging
import hmac
import hashlib

DEVICE_ID_FILE = '/etc/deviceid'
PSK_FILE = '/etc/psk'   #TODO: encrypt this

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logHandler = logging.StreamHandler()
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
logHandler.setFormatter(formatter)
logger.addHandler(logHandler)

logger.setLevel(logging.DEBUG)

class DeviceManager:
    def __init__(self):
        self.deviceId = self.readDeviceId()
        self.psk = self.readPsk()
    
    def readDeviceId(self):
        try:
            with open(DEVICE_ID_FILE, 'r', encoding="utf-8") as file:
                device_id = file.read().strip()
                logger.debug("Read device ID: %s", device_id)
                return device_id
        except FileNotFoundError:
            logger.error("Error: The file %s was not found.", DEVICE_ID_FILE)
            return ""
        except IOError:
            logger.error("Error: Unable to read the file %s.", DEVICE_ID_FILE)
            return ""
        except Exception as e:
            logger.error("An unexpected error occurred: %s", str(e))
            return ""

    def readPsk(self):
        try:
            with open(PSK_FILE, 'r', encoding="utf-8") as file:
                psk = file.read().strip()
                logger.debug("Read psk: %s", psk)
                return psk
        except FileNotFoundError:
            logger.error("Error: The file %s was not found.", PSK_FILE)
            return ""
        except IOError:
            logger.error("Error: Unable to read the file %s.", PSK_FILE)
            return ""
        except Exception as e:
            logger.error("An unexpected error occurred: %s", str(e))
            return ""

    def encryptToken(self, token):
        # Ensure the challenge token is 32 characters long
        if len(token) != 32:
            logger.error("Challenge token must be 32 characters long")
            return ""

        # Ensure the challenge token is a valid hexadecimal string
        try:
            int(token, 16)
        except ValueError:
            logger.error("Challenge token must be a valid hexadecimal string")
            return ""

        # Convert the challenge token and psk to bytes
        challenge_bytes = token.encode('utf-8')
        psk_bytes = self.psk.encode('utf-8')

        # Create the HMAC object with SHA256 as the hash function
        hmac_obj = hmac.new(psk_bytes, challenge_bytes, hashlib.sha256)

        # Get the hexadecimal digest of the signature
        signature = hmac_obj.hexdigest()

        return signature
